-- CREATE DISH service to group SOAP services
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo' ) THEN
DROP SERVICE "demo";
END IF;

CREATE SERVICE "demo" TYPE 'dish' AUTHORIZATION OFF USER DBA USING "demo";


-- CREATE SOAP services


-- SIGNON 
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/signOn' ) THEN
DROP SERVICE "demo/signOn";
END IF;

CREATE SERVICE "demo/signOn" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_SIGNON(:user, :password);

-- CUSTOMER SIGNON
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/customerSignOn' ) THEN
DROP SERVICE "demo/customerSignOn";
END IF;

CREATE SERVICE "demo/customerSignOn" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_CUSTOMER_SIGNON(:user, :password);

-- PRODUCT LIST
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/products' ) THEN
DROP SERVICE "demo/products";
END IF;

CREATE SERVICE "demo/products" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_PRODUCTS();


-- add customer
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/addCustomer' ) THEN
DROP SERVICE "demo/addCustomer";
END IF;

CREATE SERVICE "demo/addCustomer" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_ADD_CUSTOMER(
	:phone, 
    :lname, 
    :fname, 
    :address, 
    :state, 
    :company_name, 
    :zip, 
    :city);
    
-- update customer
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/updateCustomer' ) THEN
DROP SERVICE "demo/updateCustomer";
END IF;

CREATE SERVICE "demo/updateCustomer" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_UPDATE_CUSTOMER(
	:phone, 
    :lname, 
    :fname, 
    :address, 
    :state, 
    :company_name, 
    :zip, 
    :city,
    :id);    

-- get customer list
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/getCustomers' ) THEN
DROP SERVICE "demo/getCustomers";
END IF;

CREATE SERVICE "demo/getCustomers" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_GET_CUSTOMERS();


-- add order
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/addOrder' ) THEN
DROP SERVICE "demo/addOrder";
END IF;

CREATE SERVICE "demo/addOrder" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_ADD_ORDER(:cust_id,:region,:sales_rep,:order_date,:fin_code_id);

-- add order item
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/addOrderItem' ) THEN
DROP SERVICE "demo/addOrderItem";
END IF;

CREATE SERVICE "demo/addOrderItem" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_ADD_ORDER_ITEMS(:line_id,:ship_date,:quantity,:prod_id,:orderid);

-- update order
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/updateOrder' ) THEN
DROP SERVICE "demo/updateOrder";
END IF;

CREATE SERVICE "demo/updateOrder" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_UPDATE_ORDER(:cust_id,:region,:sales_rep,:order_date,:fin_code_id,:id);


-- delete order item
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/deleteOrderItem' ) THEN
DROP SERVICE "demo/deleteOrderItem";
END IF;

CREATE SERVICE "demo/deleteOrderItem" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_DELETE_ORDER_ITEMS_BY_ID(:id);


-- get orders by sales id
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/getOrderItemsBySales' ) THEN
DROP SERVICE "demo/getOrderItemsBySales";
END IF;

CREATE SERVICE "demo/getOrderItemsBySales" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_GET_ORDER_ITEMS_BY_SALES(:sales_id);


-- get order items by order id
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/getOrderItemsByOrderId' ) THEN
DROP SERVICE "demo/getOrderItemsByOrderId";
END IF;

CREATE SERVICE "demo/getOrderItemsByOrderId" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_GET_ORDER_ITEMS_BY_ID(:orderid);

-- get sales list
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/getSales' ) THEN
DROP SERVICE "demo/getSales";
END IF;


CREATE SERVICE "demo/getSales" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_GET_SALES();

-- add an order based on a given XML stream
-- an import demo to show how to use OPENXML and transaction control solution
IF EXISTS( SELECT 1
             FROM SYSWEBSERVICE
            WHERE service_name = 'demo/createOrderViaXML' ) THEN
DROP SERVICE "demo/createOrderViaXML";
END IF;


CREATE SERVICE "demo/createOrderViaXML" TYPE 'soap' AUTHORIZATION OFF USER DBA AS
CALL SP_SOAP_CREATE_ORDER_FROM_XML(:xml);


-- PROCEDURES --

IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_SIGNON' ) THEN
DROP PROCEDURE SP_SOAP_SIGNON;
END IF;

/**
 * employee sign on
 */
CREATE PROCEDURE SP_SOAP_SIGNON
(IN @user VARCHAR(32)
,IN @password VARCHAR(20))
BEGIN
	DECLARE @emp_id INTEGER;
	
    SELECT emp_id INTO @emp_id FROM EMPLOYEE WHERE emp_fname = @user AND @password = emp_fname;
    IF @emp_id IS NOT NULL THEN 
    	SELECT 
    		emp_id, emp_fname, emp_lname, STRING(NOW(*)) as token, 'Y' as status FROM employee 
    	where 
    		emp_id = @emp_id;
    ELSE 
    	SELECT NULL as emp_id, NULL as emp_fname, NULL as emp_lname, NULL as token, 'N' as status;
    END IF
END;


IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_CUSTOMER_SIGNON' ) THEN
DROP PROCEDURE SP_SOAP_CUSTOMER_SIGNON;
END IF;

/**
 * customer sign on
 */
CREATE PROCEDURE SP_SOAP_CUSTOMER_SIGNON
(IN @user VARCHAR(32)
,IN @password VARCHAR(20))
BEGIN
	DECLARE @cust_id INTEGER;
	
    SELECT id INTO @cust_id FROM CUSTOMER WHERE UPPER(lname) = UPPER(@user) AND UPPER(@password) = UPPER(fname);
    IF @cust_id IS NOT NULL THEN 
    	SELECT  
    		"phone", "lname", "fname", "address", "state", "company_name", "zip", "city", "id", STRING(NOW(*)) as token, 'Y' as status 
    	FROM 
    		customer 
    	WHERE id = @cust_id;
    ELSE 
    	SELECT NULL as "phone", NULL as "lname", NULL as "fname", NULL as "address", NULL as "state", NULL as "company_name", NULL as "zip",  NULL as city, NULL as id, NULL as token, 'N' as status;
    END IF
END;


IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_ADD_CUSTOMER' ) THEN
DROP PROCEDURE SP_SOAP_ADD_CUSTOMER;
END IF;

/**
 * send sign on request
 */
CREATE PROCEDURE SP_SOAP_ADD_CUSTOMER( 
    IN @phone VARCHAR(12), 
    IN @lname VARCHAR(20), 
    IN @fname VARCHAR(15), 
    IN @address VARCHAR(35), 
    IN @state VARCHAR(16), 
    IN @company_name VARCHAR(35), 
    IN @zip VARCHAR(10), 
    IN @city VARCHAR(20) 
    ) 
BEGIN
    INSERT INTO customer ( 
        phone, 
        lname, 
        fname, 
        address, 
        state, 
        company_name, 
        zip, 
        city, 
        id ) 
     VALUES ( 
        @phone, 
        @lname, 
        @fname, 
        @address, 
        @state, 
        @company_name, 
        @zip, 
        @city, 
        DEFAULT ); 
    SELECT @@identity AS id;
     
END;

IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_DELETE_CUSTOMER' ) THEN
DROP PROCEDURE SP_SOAP_DELETE_CUSTOMER;
END IF;

/**
 * delete customer information
 */
 
CREATE PROCEDURE SP_SOAP_DELETE_CUSTOMER ( 
    IN @id INTEGER ) 
BEGIN
    DELETE FROM customer
    WHERE
        id = @id ; 
END;

IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_GET_CUSTOMERS' ) THEN
DROP PROCEDURE SP_SOAP_GET_CUSTOMERS;
END IF;

/**
 * get list of customers
 */

CREATE PROCEDURE SP_SOAP_GET_CUSTOMERS() 
BEGIN
    SELECT "phone", "lname", "fname", "address", "state", REPLACE("company_name", '&', '&amp;') as company_name, "zip", "city", "id" 
    FROM "customer" ORDER BY id DESC;
END;



IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_ADD_ORDER' ) THEN
DROP PROCEDURE SP_SOAP_ADD_ORDER;
END IF;

/**
 * create an order entry
 */
 
CREATE PROCEDURE SP_SOAP_ADD_ORDER ( 
    IN @cust_id INTEGER, 
    IN @region VARCHAR(7), 
    IN @sales_rep INTEGER, 
    IN @order_date DATE, 
    IN @fin_code_id VARCHAR(2)
    ) 
BEGIN
    INSERT INTO sales_order ( 
        cust_id, 
        region, 
        sales_rep, 
        order_date, 
        fin_code_id, 
        id ) 
     VALUES ( 
        @cust_id, 
        @region, 
        @sales_rep, 
        @order_date, 
        @fin_code_id, 
        DEFAULT ); 
    SELECT @@identity AS id;
END;


IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_UPDATE_ORDER' ) THEN
DROP PROCEDURE SP_SOAP_UPDATE_ORDER;
END IF;

/**
 * update sales order
 */

CREATE PROCEDURE SP_SOAP_UPDATE_ORDER ( 
    IN @cust_id INTEGER, 
    IN @region CHAR(7), 
    IN @sales_rep INTEGER, 
    IN @order_date DATE, 
    IN @fin_code_id CHAR(2), 
    IN @id INTEGER ) 
BEGIN
    UPDATE sales_order
     SET 
        cust_id = @cust_id, 
        region = @region, 
        sales_rep = @sales_rep, 
        order_date = @order_date, 
        fin_code_id = @fin_code_id, 
        id = @id
    WHERE 
        id = @id ; 
    SELECT @@ROWCOUNT as id;
END;


IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_ADD_ORDER_ITEMS' ) THEN
DROP PROCEDURE SP_SOAP_ADD_ORDER_ITEMS;
END IF;

/**
 * add order items to an order entry
 */

CREATE PROCEDURE SP_SOAP_ADD_ORDER_ITEMS ( 
    IN @line_id SMALLINT, 
    IN @ship_date DATE, 
    IN @quantity INTEGER, 
    IN @prod_id INTEGER, 
    IN @orderid INTEGER ) 
BEGIN
    INSERT INTO sales_order_items ( 
        line_id, 
        ship_date, 
        quantity, 
        prod_id, 
        id ) 
     VALUES ( 
        @line_id, 
        @ship_date, 
        @quantity, 
        @prod_id, 
        @orderid ); 
     SELECT @@ROWCOUNT as id;
END;


IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_DELETE_ORDER_ITEMS_BY_ID' ) THEN
DROP PROCEDURE SP_SOAP_DELETE_ORDER_ITEMS_BY_ID;
END IF;

/**
 * delete order items
 */
CREATE PROCEDURE SP_SOAP_DELETE_ORDER_ITEMS_BY_ID ( 
    IN @id INTEGER ) 
BEGIN
    DELETE FROM sales_order_items
    WHERE
        id = @id ; 
    SELECT @@ROWCOUNT as id;
END;


IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_GET_ORDER_ITEMS_BY_SALES' ) THEN
DROP PROCEDURE SP_SOAP_GET_ORDER_ITEMS_BY_SALES;
END IF;

/**
 * get order items
 */

CREATE PROCEDURE SP_SOAP_GET_ORDER_ITEMS_BY_SALES ( 
    IN @sales_rep INTEGER ) 
BEGIN
	MESSAGE @sales_rep;
	SELECT 
		STRING("cust_id") AS cust_id, 
		"region", 
		STRING("sales_rep") AS sales_rep, 
		STRING("order_date") as order_date,
		"fin_code_id", 
		STRING("id") as id 
	FROM "sales_order" 
	WHERE sales_rep = @sales_rep 
	ORDER BY id DESC;
END;



IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_CREATE_ORDER_FROM_XML' ) THEN
DROP PROCEDURE SP_SOAP_CREATE_ORDER_FROM_XML;
END IF;

/**
 * create a sales_order and sales_order_items
 */

CREATE PROCEDURE SP_SOAP_CREATE_ORDER_FROM_XML ( 
    IN @xml LONG VARCHAR ) 
BEGIN
	DECLARE @id INTEGER;
	
	MESSAGE @xml;
	
	-- check if the order has an 'id' value
	SELECT id INTO @id FROM OPENXML( @xml, '/OBJECT/Order' ) WITH ( id int 'id');
	
	IF @id IS NULL THEN
		-- id is null, this is a new order
		-- parse the xml and insert the order information
		INSERT INTO sales_order WITH AUTO NAME
		SELECT * FROM OPENXML( @xml, '/OBJECT/Order' )
			WITH ( sales_rep int 'sales_rep', order_date varchar(20) 'order_date', 
			region varchar(30) 'region', cust_id int 'cust_id' );
		-- get order id for the autoincrement id
		SELECT @@IDENTITY INTO @id;

		MESSAGE @id;

		INSERT INTO sales_order_items WITH AUTO NAME 
		SELECT *, @id AS id FROM OPENXML( @xml, '/OBJECT/Order/orderItems/OrderItem' )
				WITH ( line_id int 'line_id', ship_date varchar(20) 'ship_date', 
				quantity int 'quantity', prod_id int 'prod_id' );
		SELECT @id as ID;	
	ELSE 
		-- update existing order
		CALL SP_SOAP_UPDATE_ORDER_FROM_XML(@id, @xml);
	END IF;
END;

IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_UPDATE_ORDER_FROM_XML' ) THEN
DROP PROCEDURE SP_SOAP_UPDATE_ORDER_FROM_XML;
END IF;

/**
 * update the sales_order and sales_order_items
 */

CREATE PROCEDURE SP_SOAP_UPDATE_ORDER_FROM_XML
(IN @orderId INTEGER
,IN @xml LONG VARCHAR ) 
BEGIN
	DECLARE @sales_rep INTEGER;
	DECLARE @order_date VARCHAR(20);
	DECLARE @region VARCHAR(30);
	DECLARE @cust_id INT;
	
	-- delete all the order items
	DELETE FROM sales_order_items WHERE id = @orderId;
	
	-- get order information from XML
	SELECT sales_rep, order_date, region, cust_id INTO
		@sales_rep,
		@order_date,
		@region,
		@cust_id
	FROM OPENXML( @xml, '/OBJECT/Order' )
			WITH ( sales_rep int 'sales_rep', order_date varchar(20) 'order_date', 
		region varchar(30) 'region', cust_id int 'cust_id' );
	
	-- update the sales_order table with order information
	UPDATE sales_order SET 
		sales_rep = @sales_rep,
		order_date = @order_date,
		region = @region,
		cust_id = @cust_id
	WHERE
		id = @orderId;
		
	-- insert new order items
	INSERT INTO sales_order_items WITH AUTO NAME 
			SELECT *, @orderId AS id FROM OPENXML( @xml, '/OBJECT/Order/orderItems/OrderItem' )
					WITH ( line_id int 'line_id', ship_date varchar(20) 'ship_date', 
					quantity int 'quantity', prod_id int 'prod_id' );
	SELECT @orderId as ID;

END;

IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_GET_ORDER_ITEMS_BY_ID' ) THEN
DROP PROCEDURE SP_SOAP_GET_ORDER_ITEMS_BY_ID;
END IF;

/**
 * get order items
 */

CREATE PROCEDURE SP_SOAP_GET_ORDER_ITEMS_BY_ID ( 
    IN @orderid INTEGER ) 
BEGIN
	SELECT 
		STRING("line_id") as line_id, 
		"ship_date", 
		STRING("quantity") AS quantity, 
		STRING("prod_id") AS prod_id, 
		STRING("id") AS id
	FROM 
		"sales_order_items" 
	WHERE 
		id = @orderid 
	ORDER BY id DESC;
END;

IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_GET_SALES' ) THEN
DROP PROCEDURE SP_SOAP_GET_SALES;
END IF;

CREATE PROCEDURE SP_SOAP_GET_SALES() 
BEGIN
    SELECT "phone", "birth_date", "bene_life_ins", "emp_fname", "street", "emp_id", "ss_number", "dept_id", "status", "salary", "sex", "emp_lname", "termination_date", "state", "bene_day_care", "start_date", "manager_id", "bene_health_ins", "city", "zip_code" FROM "employee" 
   WHERE dept_id = 200;
END;



IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_UPDATE_CUSTOMER' ) THEN
DROP PROCEDURE SP_SOAP_UPDATE_CUSTOMER;
END IF;

/**
 * update customer information
 */
CREATE PROCEDURE SP_SOAP_UPDATE_CUSTOMER ( 
    IN @phone VARCHAR(12), 
    IN @lname VARCHAR(20), 
    IN @fname VARCHAR(15), 
    IN @address VARCHAR(35), 
    IN @state VARCHAR(16), 
    IN @company_name VARCHAR(35), 
    IN @zip VARCHAR(10), 
    IN @city VARCHAR(20), 
    IN @id INTEGER ) 
BEGIN
    UPDATE customer
     SET 
        phone = @phone, 
        lname = @lname, 
        fname = @fname, 
        address = @address, 
        state = @state, 
        company_name = @company_name, 
        zip = @zip, 
        city = @city, 
        id = @id
    WHERE 
        id = @id ; 
    SELECT @@ROWCOUNT as id;
END;


IF EXISTS( SELECT 1
             FROM SYS.SYSPROCEDURE
            WHERE proc_name = 'SP_SOAP_PRODUCTS' ) THEN
DROP PROCEDURE SP_SOAP_PRODUCTS;
END IF;

/**
 * get product list
 */
CREATE PROCEDURE SP_SOAP_PRODUCTS()
BEGIN
	SELECT "color", "description", "size", string("quantity") as quantity, "name", string("unit_price") as unit_price, string("id") as id 
	FROM "product";
END;
