/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo;

/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 7-May-2003
 * Time: 2:48:58 PM
 */
import org.apache.soap.Constants;
import org.apache.soap.Fault;
import org.apache.soap.encoding.SOAPMappingRegistry;
import org.apache.soap.rpc.Call;
import org.apache.soap.rpc.Parameter;
import org.apache.soap.rpc.Response;
import org.apache.soap.util.xml.QName;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;

/**
 * The class to invoke ASA SOAP services by
 * APACHE SOAP API.
 */
public class ApacheSOAPClient {
    public static final String DEFAULT_SOAP_SERVICE_URL = "http://localhost:80/asademo/demo";
    private URL _serviceURL;
    private static final String QNAME_VALUE_SQLCODE = "tns:sqlcode";
    private static final String QNAME_VALUE_SQLROWSET = "SqlRowSet";
    private static final String QNAME_DOT_NET_NS = "http://schemas.microsoft.com/SQLServer/2001/12/SOAP/types/SqlResultStream";

    /**
     * Default constructor
     */
    public ApacheSOAPClient() {
        try {
            _serviceURL = new URL(DEFAULT_SOAP_SERVICE_URL);
        } catch (MalformedURLException e) {
        }
    }

    /**
     * Get the ASA DISH server URL
     * @return a URL, which points to
     * ASA SOAP DISH
     */
    public URL getServiceURL() {
        return _serviceURL;
    }

    /**
     * Set the ASA DISH server URL, the
     * URL must include a port number even if
     * the HTTP server is listening on 80
     * @param serviceURL
     */
    public void setServiceURL(URL serviceURL) {
        _serviceURL = serviceURL;
    }

    /**
     * Method to call any ASA soap service
     * @param methdName is the SOAP method name
     * @param params is a Vector, which holds the Parameter objects
     * @return a SQLRowSet object
     * @throws SOAPException
     * @see SQLRowSet
     */
    public SQLRowSet soapCall(String methdName, Vector params) throws SOAPException {
        try {
            Call call = new Call();
            call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
            call.setTargetObjectURI(getServiceURL().toString());
            call.setMethodName(methdName);
            if (null != params) {
                call.setParams(params);
            }
            SOAPMappingRegistry registry = new SOAPMappingRegistry();
            SQLRowSetDeserializer resultSetHelper = new SQLRowSetDeserializer();
            registry.mapTypes(Constants.NS_URI_SOAP_ENC,
                    new QName(QNAME_DOT_NET_NS, QNAME_VALUE_SQLROWSET),
                    SQLRowSet.class,
                    null,
                    resultSetHelper
            );
            registry.mapTypes(Constants.NS_URI_SOAP_ENC,
                    new QName(getServiceURL().toString(), QNAME_VALUE_SQLCODE),
                    SQLRowSet.class,
                    null,
                    resultSetHelper
            );
            call.setSOAPMappingRegistry(registry);
            // Invoke the Services
            Response resp = call.invoke(getServiceURL(), createSOAPActionHeader(methdName));
            // Check for Success
            if (!resp.generatedFault()) {
                // Extract Return value
                Parameter result = resp.getReturnValue();
                SQLRowSet value = (SQLRowSet) result.getValue();
                return value;
            }
            //  Check for Faults
            else {
                Fault f = resp.getFault();
                String faultString = f.getFaultString();
                throw new Exception(faultString);
            }
        } catch (Exception e) {
            throw new SOAPException(e);
        }
    }

    /**
     * create SOAPAction header based on service URL and
     * method name
     * @param methodName is the SOAP method name to call
     * @return a SOAPAction header value for ASA SOAP service
     * @throws Exception
     */
    private String createSOAPActionHeader(String methodName) throws Exception {
        URL url = getServiceURL();
        url = new URL(url.getProtocol(), url.getHost(), url.getPort(), url.getPath() + "/" + methodName);
        System.out.println("SOAPAction = " + url);
        return url.toString();
    }
}
