/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo;
/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 9-May-2003
 * Time: 3:12:50 PM
 */
import ianywhere.demo.types.*;
import org.apache.soap.Constants;
import org.apache.soap.rpc.Parameter;

import java.lang.reflect.Field;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

/**
 * Class to provide application SOAP service. This code
 * uses APACHE SOAP API to access SOAP service.
 * @see ApacheSOAPClient
 * @see ISoapServices
 */
public class ApacheSoapPort implements ISoapServices {
    private ApacheSOAPClient _soap = new ApacheSOAPClient();

    /**
     * Set the SOAP service URL
     * @param url is a valid URL object, which
     * holds the url to the SOAP service. The URL
     * must include a port number, even if the
     * service is on the default HTTP port.
     */
    public void setServiceURL(URL url) {
        _soap.setServiceURL(url);
    }

    /**
     * Sign on service, use employee's first name as user and
     * password
     * @param user
     * @param password
     * @return an Employee object, if the sign on succeeded,
     * the Employee object has valid user information, otherwise
     * all the properties of this object are null.
     * @see Employee
     * @throws Exception
     */
    public Employee signOn(String user, String password) throws Exception {
        Vector v = new Vector();
        Parameter p = new Parameter("user", String.class, user, Constants.NS_URI_SOAP_ENC);
        v.addElement(p);
        p = new Parameter("password", String.class, password, Constants.NS_URI_SOAP_ENC);
        v.addElement(p);
        SQLRowSet r = _soap.soapCall("signOn", v);
        Employee e = (Employee) getObject(r, Employee.class);
        return e;
    }

    /**
     * Get all the sales employee
     * @return an array of Employee objects
     * @throws Exception
     * @see Employee
     */
    public Employee[] getSales() throws Exception {
        SQLRowSet sqlRowSet = _soap.soapCall("getSales", null);
        List l = getObjects(sqlRowSet, Employee.class);
        Employee[] employees = new Employee[l.size()];
        l.toArray(employees);
        return employees;
    }

    /**
     * Get all the products
     * @return an array of Product objects
     * @throws Exception
     * @see Product
     */
    public Product[] products() throws Exception {
        SQLRowSet sqlRowSet = _soap.soapCall("products", null);
        List l = getObjects(sqlRowSet, Product.class);
        Product[] products = new Product[l.size()];
        l.toArray(products);
        return products;
    }

    /**
     * Save the Order object to the database
     * @param o is a valid Order object
     * @throws Exception
     * @see Order
     */
    public void saveOrder(Order o) throws Exception {
        Vector v = new Vector();
        Parameter param;
        param = new Parameter("cust_id", String.class, o.cust_id, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("region", String.class, o.region, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("sales_rep", String.class, o.sales_rep, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("order_date", String.class, o.order_date, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("fin_code_id", String.class, o.fin_code_id, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);

        SQLRowSet sqlRowSet = _soap.soapCall("addOrder", v);
        String sqlCode = sqlRowSet.getSqlCode();
        if (!"0".equals(sqlCode)) {
            throw new Exception("Add order failed, " + o);
        }
        o.setId(sqlRowSet.getValue(0, "id"));
    }

    /**
     * Save the order items of an order
     * @param item is a valid OrderItem object
     * @throws Exception
     * @see Order
     * @see OrderItem
     */
    public void saveOrderItem(OrderItem item) throws Exception {
        Vector v = new Vector();
        Parameter param;
        param = new Parameter("line_id", String.class, item.line_id, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("ship_date", String.class, item.ship_date, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("quantity", String.class, item.quantity, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("prod_id", String.class, item.prod_id, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("orderid", String.class, item.id, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);

        SQLRowSet sqlRowSet = _soap.soapCall("addOrderItem", v);
        String sqlCode = sqlRowSet.getSqlCode();
        if (!"0".equals(sqlCode)) {
            throw new Exception("Add order item failed, " + item);
        }
        item.setId(sqlRowSet.getValue(0, "id"));
    }

    /**
     * Get all the orders of a given sales employee
     * @param sales_id is the employe id
     * @return an array of Order objects
     * @throws Exception
     * @see Order
     */
    public Order[] getOrdersBySales(String sales_id) throws Exception {
        Vector v = new Vector();
        Parameter param = new Parameter("sales_id", String.class, sales_id, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        SQLRowSet sqlRowSet = _soap.soapCall("getOrderItemsBySales", v);
        List l = getObjects(sqlRowSet, Order.class);
        Order[] orders = new Order[l.size()];
        l.toArray(orders);
        return orders;
    }

    /**
     * Get all the order items of a give order
     * @param orderId is the order id
     * @return an array of OrderItem objects
     * @throws Exception
     * @see Order
     * @see OrderItem
     */
    public OrderItem[] getOrderItemsByOrderId(String orderId) throws Exception {
        Vector v = new Vector();
        Parameter param = new Parameter("orderid", String.class, orderId, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        SQLRowSet sqlRowSet = _soap.soapCall("getOrderItemsByOrderId", v);
        List l = getObjects(sqlRowSet, OrderItem.class);
        OrderItem[] ordersItems = new OrderItem[l.size()];
        l.toArray(ordersItems);
        return ordersItems;
    }

    /**
     * Add a customer contact information to the database
     * @param c is a valid Customer object
     * @throws Exception
     * @see Customer
     */
    public void addCustomer(Customer c) throws Exception {
        Vector v = new Vector();
        Parameter param;

        param = new Parameter("phone", String.class, c.phone, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("lname", String.class, c.lname, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("fname", String.class, c.fname, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("address", String.class, c.address, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("state", String.class, c.state, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("company_name", String.class, c.company_name, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("zip", String.class, c.zip, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("city", String.class, c.city, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);

        SQLRowSet sqlRowSet = _soap.soapCall("addCustomer", v);
        String sqlCode = sqlRowSet.getSqlCode();
        if (!"0".equals(sqlCode)) {
            throw new Exception("Add customer failed, " + c);
        }
        c.setId(sqlRowSet.getValue(0, "id"));
    }

    /**
     * Create an order by using an XML stream. All the order
     * information are in the XML. In this demo, the XMLSerializer
     * serializes the Order object to XML stream.
     * @param xml is a serialized XML string of an Order object
     * @return the order id in the database
     * @throws Exception
     * @see Order
     * @see OrderItem
     * @see XMLFriendly
     * @see XMLSerializer
     */
    public int createOrderViaXml(String xml) throws Exception {
        Vector v = new Vector();
        Parameter param = new Parameter("xml", String.class, xml, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        SQLRowSet sqlRowSet = _soap.soapCall("createOrderViaXML", v);
        if (!"0".equals(sqlRowSet.getSqlCode())) {
            throw new Exception("Transaction failed!");
        }
        String value = sqlRowSet.getValue(0,"id");
        return Integer.parseInt(value);
    }

    /**
     * Update an order information
     * @param order is a valid Order object
     * @throws Exception
     * @see Order
     */
    public void updateOrder(Order order) throws Exception {
        Vector v = new Vector();
        Parameter param;

        param = new Parameter("cust_id", String.class, order.cust_id, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("region", String.class, order.region, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("sales_rep", String.class, order.sales_rep, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("order_date", String.class, order.order_date, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("fin_code_id", String.class, order.fin_code_id, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("id", String.class, order.id, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);

        SQLRowSet sqlRowSet = _soap.soapCall("updateOrder", v);
        String sqlCode = sqlRowSet.getSqlCode();
        if (!"0".equals(sqlCode)) {
            throw new Exception("Update order item failed, " + order);
        }
    }

    /**
     * Delete all the order items of a given order id
     * @param orderId is an existing order id
     * @throws Exception
     * @see OrderItem
     * @see Order
     */
    public void deleteOrderItems(String orderId) throws Exception {
        Vector v = new Vector();
        Parameter param = new Parameter("id", String.class, orderId, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        SQLRowSet sqlRowSet = _soap.soapCall("deleteOrderItem", v);
        if (!"0".equals(sqlRowSet.getSqlCode())) {
            throw new Exception("Failed to delete order items, order id = " + orderId);
        }
    }

    /**
     * Update a customer information
     * @param c is a valid customer object
     * @throws Exception
     * @see Customer
     */
    public void updateCustomer(Customer c) throws Exception {
        Vector v = new Vector();
        Parameter param;

        param = new Parameter("phone", String.class, c.phone, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("lname", String.class, c.lname, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("fname", String.class, c.fname, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("address", String.class, c.address, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("state", String.class, c.state, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("company_name", String.class, c.company_name, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("zip", String.class, c.zip, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("city", String.class, c.city, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);
        param = new Parameter("id", String.class, c.id, Constants.NS_URI_SOAP_ENC);
        v.addElement(param);

        SQLRowSet sqlRowSet = _soap.soapCall("updateCustomer", v);
        String sqlCode = sqlRowSet.getSqlCode();
        if (!"0".equals(sqlCode)) {
            throw new Exception("Update customer failed, " + c);
        }
    }

    /**
     * Get all the customers
     * @return an array of Customer objects
     * @throws Exception
     * @see Customer
     */
    public Customer[] getCustomers() throws Exception {
        SQLRowSet sqlRowSet = _soap.soapCall("getCustomers", null);
        List l = getObjects(sqlRowSet, Customer.class);
        Customer[] customers = new Customer[l.size()];
        l.toArray(customers);
        return customers;
    }

    /**
     * Get an object from ASA SOAP response. In this demo,
     * the business Java classes, such as Customer, Employee,
     * Order, OrderItem, Product have property names, which are
     * same as the corresponding table column names. This method
     * uses Reflection API to get the property names then it uses
     * the property name as the column name to get value from
     * the response. This is the way to populate all the properties.
     *
     * @param sqlRowSet is the deserialized response from SOAP service
     * @param c is the Java class type to be populated
     * @return a java Object, whose class type is c
     * @throws Exception
     * @see SQLRowSet
     */
    private Object getObject(SQLRowSet sqlRowSet, Class c) throws Exception {
        Field[] fields = c.getFields();
        Object o = c.newInstance();
        for (int f = 0; f < fields.length; f++) {
            Field field = fields[f];
            String value = sqlRowSet.getValue(0, field.getName());
            if (null != value) {
                field.set(o, value);
            }
        }
        return o;
    }

    /**
     * Get a List of objects from ASA SOAP response. In this demo,
     * the business Java classes, such as Customer, Employee,
     * Order, OrderItem, Product have property names, which are
     * same as the corresponding table column names. This method
     * uses Reflection API to get the property names then it uses
     * the property name as the column name to get value from
     * the response. This is the way to populate all the properties.
     *
     * @param sqlRowSet is the deserialed SOAP response from ASA
     * @param c is the object Java class type
     * @return a list of objects, whose class type is c
     * @throws Exception
     */
    private List getObjects(SQLRowSet sqlRowSet, Class c) throws Exception {
        List l = new ArrayList();
        int columnCount = sqlRowSet.getColumnCount();
        int rowCount = sqlRowSet.getRowCount();
        for (int r = 0; r < rowCount; r++) {
            Object o = c.newInstance();
            for (int col = 0; col < columnCount; col++) {
                String columnName = sqlRowSet.getColumnName(col);
                Field field = c.getField(columnName);
                if (null != field) {
                    field.set(o, sqlRowSet.getValue(r, col));
                }
            }
            l.add(o);
        }
        return l;
    }
}
