/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo;

/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 9-May-2003
 * Time: 3:20:57 PM
 */

import ianywhere.demo.types.*;

import java.net.URL;

/**
 * Interface to define an abstract layer between
 * application service and SOAP service call so
 * that user can switch to other SOAP APIs by providing
 * different implementation classes. In this demo, we
 * provide an implementation class based on Apache SOAP.
 */
public interface ISoapServices {

    /**
     * Set the SOAP service URL
     * @param url is a valid URL object, which
     * holds the url to the SOAP service. The URL
     * must include a port number, even if the
     * service is on the default HTTP port.
     */    
    void setServiceURL(URL url);

    /**
     * Sign on service, use employee's first name as user and
     * password
     * @param user
     * @param password
     * @return an Employee object, if the sign on succeeded,
     * the Employee object has valid user information, otherwise
     * all the properties of this object are null.
     * @see Employee
     * @throws Exception
     */
    Employee signOn(String user, String password) throws Exception;

    /**
     * Get all the sales employee
     * @return an array of Employee objects
     * @throws Exception
     * @see Employee
     */
    Employee[] getSales() throws Exception;

    /**
     * Get all the products
     * @return an array of Product objects
     * @throws Exception
     * @see Product
     */
    Product[] products() throws Exception;

    /**
     * Save the Order object to the database
     * @param o is a valid Order object
     * @throws Exception
     * @see Order
     */
    void saveOrder(Order o) throws Exception;

    /**
     * Save the order items of an order
     * @param item is a valid OrderItem object
     * @throws Exception
     * @see Order
     * @see OrderItem
     */
    void saveOrderItem(OrderItem item) throws Exception;

    /**
     * Get all the orders of a given sales employee
     * @param sales_id is the employe id
     * @return an array of Order objects
     * @throws Exception
     * @see Order
     */
    Order[] getOrdersBySales(String sales_id) throws Exception;

    /**
     * Get all the order items of a give order
     * @param orderId is the order id
     * @return an array of OrderItem objects
     * @throws Exception
     * @see Order
     * @see OrderItem
     */
    OrderItem[] getOrderItemsByOrderId(String orderId) throws Exception;

    /**
     * Add a customer contact information to the database
     * @param c is a valid Customer object
     * @throws Exception
     * @see Customer
     */
    void addCustomer(Customer c) throws Exception;

    /**
     * Create an order by using an XML stream. All the order
     * information are in the XML. In this demo, the XMLSerializer
     * serializes the Order object to XML stream.
     * @param xml is a serialized XML string of an Order object
     * @return the order id in the database
     * @throws Exception
     * @see Order
     * @see OrderItem
     * @see XMLFriendly
     * @see XMLSerializer
     */
    int createOrderViaXml(String xml) throws Exception;

    /**
     * Update an order information
     * @param order is a valid Order object
     * @throws Exception
     * @see Order
     */
    void updateOrder(Order order) throws Exception;

    /**
     * Delete all the order items of a given order id
     * @param orderId is an existing order id
     * @throws Exception
     * @see OrderItem
     * @see Order
     */
    void deleteOrderItems(String orderId) throws Exception;

    /**
     * Update a customer information
     * @param c is a valid customer object
     * @throws Exception
     * @see Customer
     */
    void updateCustomer(Customer c) throws Exception;

    /**
     * Get all the customers
     * @return an array of Customer objects
     * @throws Exception
     * @see Customer
     */
    Customer[] getCustomers() throws Exception;
}
