/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo.proxy;

import javax.swing.*;
import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Vector;
import java.awt.*;

/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 27-Mar-2003
 * Time: 4:38:11 PM
 */
public class Proxy implements Runnable {
    private int _port = 80;
    private int _serverPort = 8080;
    private String _serverHost = "localhost";
    private ServerSocket _serverSocket;
    private Vector _threads = new Vector();
    private JFrame _window;
    private JTextArea _outputText = new JTextArea();
    private static Proxy _instance = new Proxy();

    public static void main(String[] args) throws Exception {
        System.out.println("Usage: proxy SOAP_SERVER_HOST SOAP_SERVER_PORT");
        System.out.println("Default SOAP_SERVER_HOST=localhost, SOAP_SERVER_PORT=8080");
        Proxy p = new Proxy();
        if (args.length == 2) {
            p.setServerHost(args[0]);
            p.setServerPort(Integer.parseInt(args[1]));
        }
        p.start();
    }

    public static Proxy getInstance() {
        return _instance;
    }

    private Proxy() {
    }

    public int getServerPort() {
        return _serverPort;
    }

    public void setServerPort(int serverPort) {
        _serverPort = serverPort;
    }

    public String getServerHost() {
        return _serverHost;
    }

    public void setServerHost(String serverHost) {
        _serverHost = serverHost;
    }

    public int getPort() {
        return _port;
    }

    public void setPort(int port) {
        _port = port;
    }

    public void start() throws Exception {
        if (null == _serverSocket) {
            _serverSocket = new ServerSocket(getPort(), 2048, InetAddress.getLocalHost());
            Thread t = new Thread(this);
            t.start();
            logn("Proxy started at " + _serverSocket.getInetAddress() + ", port " + _serverSocket.getLocalPort());
            logn("Redirecting requests to " + getServerHost() + ":" + getServerPort());
        }
    }

    public void shutDown() {
        for (int i = 0; i < _threads.size(); i++) {
            Controllable c = (Controllable) _threads.elementAt(i);
            c.shutdown();
        }
        try {
            _serverSocket.close();
        } catch (IOException ignore) {
        }
    }

    public void run() {
        while (true) {
            try {
                Socket incomingSocket = _serverSocket.accept();
                try {
                    Socket soapSocket = new Socket(getServerHost(), getServerPort());
                    WorkerThread wt = new WorkerThread(this, incomingSocket, soapSocket);
                    wt.setRedirectHost(getServerHost());
                    wt.setRedirectPort(getServerPort());
                    wt.setIncoming(true);
                    _threads.add(wt);

                    WorkerThread wt1 = new WorkerThread(this, soapSocket, incomingSocket);
                    _threads.add(wt1);
                    wt.start();
                    wt1.start();
                } catch (Exception e) {
                    logn(e.toString());
                }
            } catch (Exception e) {
                logn(e.toString());
                try {
                    _serverSocket.close();
                } catch (IOException ignore) {
                }
                return;
            }
        }
    }

    public void showConsole() {
        if (_window == null) {
            createWindow();
        }
        _window.setVisible(true);
    }

    public void closeConsole() {
        if (null != _window) {
            _window.hide();
        }
    }

    public void log(String text) {
        _outputText.append(text);
        _outputText.setCaretPosition(_outputText.getDocument().getLength());
    }

    public void logn(String text) {
        log(text);
        log("\n");
    }

    private void createWindow() {
        JScrollPane jsp = new JScrollPane(_outputText);
        _window = new JFrame("SOAP Proxy console");
        Container contentPane = _window.getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(jsp);
        _outputText.setWrapStyleWord(true);
        _outputText.setLineWrap(true);
        _window.setSize(400, 200);
    }
}
