/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo.types;
/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 27-Mar-2003
 * Time: 2:35:07 PM
 */
import ianywhere.demo.ISoapServices;
import ianywhere.demo.ui.Application;

/**
 * Class to represent an order, the
 * property names are same as database
 * table column names.
 */
public class Order implements XMLFriendly {
    public String cust_id,
    region,
    sales_rep,
    order_date,
    fin_code_id,
    id;

    private OrderItem[] _orderItems;

    /**
     * Get all the orders of a given sales
     * @param salesId
     * @return an array of Order objects
     * @throws Exception
     * @see Order
     */
    public static Order[] getOrdersBySales(String salesId) throws Exception {
        Application app = Application.getInstance();
        ISoapServices sc = app.getSoapClient();
        Order[] orders = sc.getOrdersBySales(salesId);
        return orders;
    }

    /**
     * Get order items of this order
     * @return an array of OrderItem objects
     * @throws Exception
     * @see OrderItem
     */
    public OrderItem[] getOrderItems() throws Exception {
        if (null == _orderItems) {
            Application app = Application.getInstance();
            ISoapServices sc = app.getSoapClient();
            _orderItems = sc.getOrderItemsByOrderId(id);
        }
        return _orderItems;
    }

    /**
     * Return property names in an array,
     * these properties will go to the XML stream
     * @return an array of property names
     */
    public String[] getFriendlyFields() {
        return new String[]{
            "cust_id", "region", "sales_rep",
            "order_date", "fin_code_id", "id", "orderItems"};
    };

    /**
     * Check if there are order items
     * @return true if there are some items
     */
    public boolean hasItems() {
        return null != _orderItems && _orderItems.length > 0;
    }

    /**
     * check if it is a new order, which doesn't have
     * order id
     * @return true if it is a new order
     */
    public boolean isNewOrder() {
        return null == id || id.trim().length() == 0;
    }

    /**
     * Set the order items
     * @param orderItems
     */
    public void setOrderItems(OrderItem[] orderItems) {
        _orderItems = orderItems;
    }

    /**
     * Get customer id
     * @return customer id
     */
    public String getCust_id() {
        return cust_id;
    }

    /**
     * Set customer id
     * @param cust_id
     */
    public void setCust_id(String cust_id) {
        this.cust_id = cust_id;
    }

    /**
     * Get region
     * @return the region
     */
    public String getRegion() {
        return region;
    }

    /**
     * Set the region
     * @param region
     */
    public void setRegion(String region) {
        this.region = region;
    }

    /**
     * Get sales
     * @return sales id
     */
    public String getSales_rep() {
        return sales_rep;
    }

    /**
     * Set sales id
     * @param sales_rep
     */
    public void setSales_rep(String sales_rep) {
        this.sales_rep = sales_rep;
    }

    /**
     * Get order date
     * @return order date
     */
    public String getOrder_date() {
        return order_date;
    }

    /**
     * Set order date
     * @param order_date
     */
    public void setOrder_date(String order_date) {
        this.order_date = order_date;
    }

    /**
     * Get fin code
     * @return fin code
     */
    public String getFin_code_id() {
        return fin_code_id;
    }

    /**
     * Set fin code
     * @param fin_code_id
     */
    public void setFin_code_id(String fin_code_id) {
        this.fin_code_id = fin_code_id;
    }

    /**
     * Get order id
     * @return order id
     */
    public String getId() {
        return id;
    }

    /**
     * Set order id
     * @param id
     */
    public void setId(String id) {
        this.id = id;
    }

    /**
     * Return a string presentation of this order
     * @return a string shows order information
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("[ id = " + id);
        sb.append(", ");
        sb.append("cust_id = " + cust_id);
        sb.append(", ");
        sb.append("region = " + region);
        sb.append(", ");
        sb.append("sales_rep = " + sales_rep);
        sb.append(", ");
        sb.append("order_date = " + order_date);
        sb.append(", ");
        sb.append("fin_code_id = " + fin_code_id);
        sb.append(" ]");
        try {
            _orderItems = getOrderItems();
        } catch (Exception e) {
//            System.out.println(e);
            _orderItems = null;
        }
        if (hasItems()) {
            sb.append("\n     ");
            for (int i = 0; i < _orderItems.length; i++) {
                OrderItem orderItem = _orderItems[i];
                sb.append(orderItem.toString());
                if (i + 1 < _orderItems.length) {
                    sb.append(", ");
                }
            }
        }
        return sb.toString();
    }

    /**
     * Save the order, if the order has an order id
     * then this method updates the order
     * @param o is a valid Order object
     * @throws Exception
     */
    public static void saveOrder(Order o) throws Exception {
        if (o.isNewOrder()) {
            ISoapServices soapClient = Application.getInstance().getSoapClient();
            soapClient.saveOrder(o);
            saveOrderItems(o);
        } else {
            updateOrder(o);
        }
    }

    /**
     * Save the order items to sales_order_items table
     * @param o is a valid Order object, which has
     * order items
     * @throws Exception
     */
    private static void saveOrderItems(Order o) throws Exception {
        ISoapServices sc = Application.getInstance().getSoapClient();
        OrderItem[] orderItems = o.getOrderItems();
        for (int i = 0; i < orderItems.length; i++) {
            OrderItem orderItem = orderItems[i];
            orderItem.setId(o.getId());
            orderItem.setLine_id(String.valueOf(i + 1));
            sc.saveOrderItem(orderItem);
        }
    }

    /**
     * Update an order
     * @param o is a valid Order object
     * @throws Exception
     */
    private static void updateOrder(Order o) throws Exception {
        ISoapServices sc = Application.getInstance().getSoapClient();
        sc.updateOrder(o);
        sc.deleteOrderItems(o.getId());
        saveOrderItems(o);
    }

    /**
     * Update the order via XML stream. The XML is
     * generated by XMLSerializer
     * @param order
     * @return the order id in database
     * @throws Exception
     */
    public static int saveViaXml(Order order) throws Exception {
        OrderItem[] orderItems = order.getOrderItems();
        for (int i = 0; i < orderItems.length; i++) {
            OrderItem orderItem = orderItems[i];
            orderItem.line_id = String.valueOf(i+1);
        }
        ISoapServices sc = Application.getInstance().getSoapClient();
        XMLSerializer xs = new XMLSerializer();
        String xml = xs.toXML(order);
        System.out.println("xml = " + xml);
        return sc.createOrderViaXml(xml);
    }
}
