/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo.types;
/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 27-Mar-2003
 * Time: 2:35:13 PM
 */
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Class to represent an order item, the
 * property names are same as database
 * table column names.
 */
public class OrderItem implements XMLFriendly {
    public String line_id,
    ship_date,
    quantity,
    prod_id,
    id;
    private Product _product;
    private static final String DATE_FORMAT_PATTERN = "yyyy-MM-dd";
    private static SimpleDateFormat _formatter = new SimpleDateFormat(DATE_FORMAT_PATTERN);

    /**
     * default constructor
     */
    public OrderItem() {
        try {
            Product[] products = Product.getProducts();
            setProduct(products[0]);
        } catch (Exception e) {
            System.out.println("e = " + e);
        }
        quantity = "1";
        Date aValue = new Date();
        ship_date = _formatter.format(aValue);
    }

    /**
     * Return property names in an array,
     * these properties will go to the XML stream
     * @return an array of property names
     */
    public String[] getFriendlyFields() {
        return new String[]{
            "line_id", "ship_date", "quantity", "prod_id", "id", "product"
        };
    }

    /**
     * Get the product
     * @return a Product object
     * @see Product
     */
    public Product getProduct() {
        return _product;
    }

    /**
     * Set the order product
     * @param product
     */
    public void setProduct(Product product) {
        if (null != product) {
            _product = product;
            setProd_id(product.getId());
        }
    }

    /**
     * get the line id
     * @return
     */
    public String getLine_id() {
        return line_id;
    }

    /**
     * Set the line id
     * @param line_id
     */
    public void setLine_id(String line_id) {
        this.line_id = line_id;
    }

    /**
     * Get the ship date
     * @return the ship date
     */
    public String getShip_date() {
        return ship_date;
    }

    /**
     * Set the ship date
     * @param ship_date
     */
    public void setShip_date(String ship_date) {
        this.ship_date = ship_date;
    }

    /**
     * Get order quantity
     * @return the quantity
     */
    public String getQuantity() {
        return quantity;
    }

    /**
     * Set the order quantity
     * @param quantity
     */
    public void setQuantity(String quantity) {
        this.quantity = quantity;
    }

    /**
     * Get the product id
     * @return product id
     */
    public String getProd_id() {
        return prod_id;
    }

    /**
     * Set the product id
     * @param prod_id
     */
    public void setProd_id(String prod_id) {
        this.prod_id = prod_id;
    }

    /**
     * Get order id
     * @return order id
     */
    public String getId() {
        return id;
    }

    /**
     * Set the order id
     * @param id
     */
    public void setId(String id) {
        this.id = id;
    }

    public String toString() {
        return getProd_id() + ", " + getQuantity();
    }
}
