/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo.types;

/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 30-Mar-2003
 * Time: 9:38:00 AM
 */

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Array;

/**
 * This is a utility class to
 * convert a Java object to an XML stream. The
 * rule is that the Java object must implement
 * XMLFriendly interface to indicate which propertie's
 * value goes the XML stream. For example:
 * <pre>
 * class foo {
 *    int a;
 *    String b;
 *
 *    public int getA() { return a; }
 *    public String getB() ( return b; }
 *
 *    public String[] getFriendlyFields() {
 *       return new String[] { "a", "b" };
 *    }
 * </pre>
 * If a=1 and b="abc" then the XML looks like:
 * <OBJECT><foo><a>1</a><b>abc</b></foo></OBJECT>.
 * The processing will drill down all the properties,
 * as long as they implement the XMLFriendly interface.
 * @see XMLFriendly
 */
public class XMLSerializer {
    private DocumentBuilderFactory _factory;
    private DocumentBuilder _builder;
    private StringBuffer _buf = new StringBuffer();
    private static final String ROOT_TAG = "OBJECT";

    /**
     * method for testing
     * @param args
     * @throws Exception
     */
    public static void main(String[] args) throws Exception {
        Order o = new Order();
        o.id = "1";
        o.sales_rep = "David Huo";
        o.order_date = "2003-3-3";
        o.cust_id = "768";
        o.region = "Central";
        o.fin_code_id = "fin_code_xxx";
        OrderItem oi1 = new OrderItem();
        oi1.id = "1";
        oi1.line_id = "1";
        oi1.prod_id = "300";
        oi1.quantity = "10";
        oi1.ship_date = "2003-2-2";

        C c = new C();
        c._order = o;

        Product p = new Product();
        p.id = "300";
        p.color = "Read";
        p.description = "T-shirt in red";
        p.unit_price = "10";
        p.name = "T-shirt";
        p.size = "XL";

        oi1.setProduct(p);

        OrderItem oi2 = new OrderItem();
        oi2.id = "1";
        oi2.line_id = "2";
        oi2.prod_id = "301";
        oi2.quantity = "20";
        oi2.ship_date = "2003-2-3";

        OrderItem[] items = new OrderItem[]{oi1, oi2};
        o.setOrderItems(items);

        XMLSerializer xs = new XMLSerializer();
        Document doc = xs.serialize(c);
        xs.printDocument(doc, new OutputStreamWriter(System.out));
    }

    /**
     * Constructor
     * @throws Exception
     */
    public XMLSerializer() throws Exception {
        _factory = DocumentBuilderFactory.newInstance();
        _builder = _factory.newDocumentBuilder();
    }

    /**
     * Convert the XML friendly objects
     * to DOM object, including the embedded objects
     * @param x
     * @return a DOM object
     * @throws Exception
     */
    public Document serialize(XMLFriendly x) throws Exception {
        Document doc = _builder.newDocument();
        Element root = doc.createElement(ROOT_TAG);
        serialize(x, root, doc);
        doc.appendChild(root);
        return doc;
    }

    /**
     * Convert the XML friendly objects
     * to XML stream, including the embedded objects
     * @param x
     * @return an XML string
     * @throws Exception
     */
    public String toXML(XMLFriendly x) throws Exception {
        StringWriter sw = new StringWriter();
        Document doc = serialize(x);
        printDocument(doc, sw);
        String rawXml = sw.toString();
        return rawXml;
    }

    /**
     * Print the DOM document to a Writer object
     * @param doc
     * @param out
     * @throws Exception
     */
    private void printDocument(Document doc, Writer out) throws Exception {
        Transformer transformer = TransformerFactory.newInstance().newTransformer();
        transformer.transform(new DOMSource(doc), new StreamResult(out));
    }

    /**
     * Drill down to all the properties
     * @param x
     * @param tag
     * @param doc
     * @throws NoSuchMethodException
     * @throws IllegalAccessException
     * @throws InvocationTargetException
     */
    private void serialize(XMLFriendly x, Element tag, Document doc) throws NoSuchMethodException, IllegalAccessException, InvocationTargetException {
        String[] friendlyFields = x.getFriendlyFields();
        Class aClass = x.getClass();
        String className = getShortName(aClass);
        System.out.println("className = " + className);
        Element tagName = doc.createElement(className);
        tag.appendChild(tagName);
        for (int i = 0; i < friendlyFields.length; i++) {
            String ff = friendlyFields[i];
            char[] chars = ff.toCharArray();
            chars[0] = Character.toUpperCase(chars[0]);
            _buf.setLength(0);
            _buf.append("get");
            _buf.append(chars);
            String methodName = _buf.toString();
//            System.out.println("methodName = " + methodName);
            Method method = aClass.getMethod(methodName, null);
            Object v = method.invoke(x, null);
            Element fieldTag = null;
            if (null != v) {
                Class retClass = v.getClass();
                if (retClass.isArray()) {
                    fieldTag = doc.createElement(ff);
                    int length = Array.getLength(v);
                    for (int j = 0; j < length; j++) {
                        Object o = Array.get(v, j);
                        if (o instanceof XMLFriendly) {
                            XMLFriendly xf = (XMLFriendly) o;
                            serialize(xf, fieldTag, doc);
                        } else {
                            Element value;
                            if(ff.endsWith("s")) {
                                String vName = ff.substring(0,ff.length()-1);
                                value = doc.createElement(vName);
                            } else {
                                value = doc.createElement("VALUE");
                            }
                            value.appendChild(doc.createTextNode(o.toString()));
                            fieldTag.appendChild(value);
                        }
                    }
                    tagName.appendChild(fieldTag);
                } else if (v instanceof XMLFriendly) {
                    XMLFriendly xf = (XMLFriendly) v;
                    serialize(xf, tagName, doc);
                } else {
                    fieldTag = doc.createElement(ff);
                    fieldTag.appendChild(doc.createTextNode(v.toString()));
                    tagName.appendChild(fieldTag);
                }
            }
        }
    }

    /**
     * Remove the package part from the
     * class name
     * @param c
     * @return the class name without package name
     */
    private String getShortName(Class c) {
        String name = c.getName();
        int bc;
        bc = name.lastIndexOf('$');
        if (bc > 0) {
            return name.substring(bc + 1, name.length());
        } else {
            bc = name.lastIndexOf('.');
            if (bc > 0) {
                return name.substring(bc + 1, name.length());
            }
            return name;
        }
    }

    private static class C implements XMLFriendly {
        private Order _order;
        int[] _values = new int[]{1, 2, 3, 4};

        /**
         * Return property names in an array,
         * these properties will go to the XML stream
         * @return an array of property names
         */
        public String[] getFriendlyFields() {
            return new String[]{"values", "order"};
        }

        public Order getOrder() {
            return _order;
        }

        public int[] getValues() {
            return _values;
        }
    }
}
