/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo.ui;

import ianywhere.demo.ApacheSOAPClient;
import ianywhere.demo.ApacheSoapPort;
import ianywhere.demo.ISoapServices;
import ianywhere.demo.SOAPException;
import ianywhere.demo.proxy.Proxy;
import ianywhere.demo.types.Customer;
import ianywhere.demo.types.Employee;
import ianywhere.demo.types.Order;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 27-Mar-2003
 * Time: 11:11:35 PM
 */
public class Application extends PreferedSizeFrame {
    public static Application _instance;
    private static URL _serviceURL;
    private static final String TITLE = "ASA 9.0 SOAP Service Demo Application";
    private static final Dimension MIN_SIZE = new Dimension(640, 400);
    private SignOnPanel _signOn;
    private Employee _employee;
    private ISoapServices _soap = null;
    private OrderEntry _uiOrderEntry;
    private CustomerView _customersView;
    private CustomerEntry _uiAddCustomer = new CustomerEntry();
    private HelpPanel _uiHelp = new HelpPanel();
    private OrderView _uiOrderView = new OrderView();

    static {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Throwable e) {
        }
    }

    public static void main(String[] args) {
        if (args.length > 0) {
            if (args[0].equals("?")) {
                System.out.println("Usage java Application [SOAP Service URL].");
                System.out.println("Default SOAP Service URL is " + ApacheSOAPClient.DEFAULT_SOAP_SERVICE_URL);
                System.exit(0);
            } else {
                try {
                    _serviceURL = new URL(args[0]);
                    if(_serviceURL.getPort() == -1) {
                        System.out.println("Please specify port number in the URL.");
                        System.exit(-2);
                    }
                } catch (MalformedURLException e) {
                    System.out.println("Invalid URL " + args[0]);
                    System.exit(-1);
                }
            }
        }
        new Application();
    }

    public static Application getInstance() {
        return _instance;
    }

    private Application() {
        super(MIN_SIZE);
        _instance = this;
        initialize();
    }

    private SignOnPanel getSignOnUserInterface() throws Exception {
        if (null == _signOn) {
            _signOn = new SignOnPanel();
        }
        return _signOn;
    }

    public void newOrder() {
        if (_uiOrderEntry == null) {
            try {
                _uiOrderEntry = new OrderEntry();
            } catch (Exception e) {
                Application.reportException(e);
                return;
            }
        }
        _uiOrderEntry.newOrder();
        setUserInterface(_uiOrderEntry);
    }

    public ISoapServices getSoapClient() throws Exception {
        if (null == _soap) {
            _soap = new ApacheSoapPort();
            if( null != _serviceURL ) {
                _soap.setServiceURL(_serviceURL);
            }
        }
        return _soap;
    }

    public Employee getEmployee() {
        return _employee;
    }

    public void setEmployee(Employee employee) {
        System.out.println("Application.setEmployee");
        _employee = employee;
        if (null != employee) {
            setTitle(TITLE + " - " + _employee);
        }
    }

    public void editCustomer(Customer customer) {
        setUserInterface(_uiAddCustomer);
        _uiAddCustomer.setCustomer(customer);
    }

    public void editOrder(Order order) {
        newOrder();
        try {
            _uiOrderEntry.setOrder(order);
        } catch (Exception e) {
            reportException(e);
        }
    }

    public void setUserInterface(Component userInterface) {
        removeUserInterface();
        Container contentPane = getContentPane();
        contentPane.add(userInterface, BorderLayout.CENTER);
        getContentPane().validate();
        repaint();
    }

    public static void reportException(Throwable t) {
        String title = "Generic Exception";
        String message = t.getMessage();
        t.printStackTrace();
        if (t instanceof SOAPException) {
            SOAPException soapException = (SOAPException) t;
            title = "SOAP error";
            message = soapException.getVendorException().toString();
        }
        JOptionPane.showMessageDialog(Application.getInstance(),
                message, title, JOptionPane.OK_OPTION);
    }

    private void initialize() {
        setTitle(TITLE);
        Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());
        setupMenu();
        setSize(MIN_SIZE);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        contentPane.add(StatusBar.getInstance(), BorderLayout.SOUTH);
        showWelcome();
        setVisible(true);
    }

    private void showWelcome() {
        _uiHelp.showWelcome();
        setUserInterface(_uiHelp);
    }

    private void setupMenu() {
        JMenuBar mbar = new JMenuBar();
        JMenu mFile = new JMenu("File");
        JMenu mHelp = new JMenu("Help");
        mFile.setMnemonic('F');
        mHelp.setMnemonic('H');
        JMenuItem miAbout = new JMenuItem("About");
        JMenuItem miConsole = new JMenuItem("Show Proxy Console");
        miConsole.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Proxy.getInstance().showConsole();
            }
        });
        miAbout.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                showWelcome();
            }
        });
        mHelp.add(miAbout);
        mHelp.addSeparator();
        mHelp.add(miConsole);
        JMenuItem miExit = new JMenuItem("Exit");
        JMenuItem miLogon = new JMenuItem("Logon");
        miLogon.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, KeyEvent.CTRL_MASK));
        miLogon.setMnemonic('o');
        mFile.add(miLogon);
        miLogon.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    SignOnPanel ui = getSignOnUserInterface();
                    ui.showDialog(Application.getInstance());
                } catch (Exception e1) {
                    Application.reportException(e1);
                }
            }
        });
        miExit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });
        JMenuItem miOrder = new JMenuItem("New order");
        miOrder.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, KeyEvent.CTRL_MASK));
        miOrder.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (checkSignOn()) {
                    newOrder();
                }
            }
        });
        JMenuItem miViewOrders = new JMenuItem("View order list");
        miViewOrders.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, KeyEvent.CTRL_MASK));
        miViewOrders.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (checkSignOn()) {
                    viewOrderList();
                }
            }
        });
        miViewOrders.setMnemonic('V');
        miOrder.setMnemonic('E');
        miExit.setMnemonic('x');
        miExit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, KeyEvent.CTRL_MASK));
        mFile.add(miOrder);
        mFile.add(miViewOrders);
        mFile.addSeparator();
        JMenuItem miCustomers = new JMenuItem("View Customer List");
        JMenuItem miNewCustomer = new JMenuItem("Add Customer");
        miCustomers.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, KeyEvent.CTRL_MASK));
        miNewCustomer.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, KeyEvent.CTRL_MASK));
        miNewCustomer.setMnemonic('A');
        miCustomers.setMnemonic('L');
        miNewCustomer.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setUserInterface(_uiAddCustomer);
            }
        });
        miCustomers.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (checkSignOn()) {
                    if (null == _customersView) {
                        _customersView = new CustomerView();
                    }
                    setUserInterface(_customersView);
                    try {
                        _customersView.retrieve();
                    } catch (Exception e1) {
                        reportException(e1);
                    }
                }
            }
        });
        mFile.add(miCustomers);
        mFile.add(miNewCustomer);
        mFile.addSeparator();
        mFile.add(miExit);
        mbar.add(mFile);
        mbar.add(mHelp);
        setJMenuBar(mbar);
    }

    private void viewOrderList() {
        setUserInterface(_uiOrderView);
        try {
            _uiOrderView.retrieve(getEmployee());
        } catch (Exception e) {
            reportException(e);
        }
    }

    private void removeUserInterface() {
        Component[] components = getContentPane().getComponents();
        for (int i = 0; i < components.length; i++) {
            Component component = components[i];
            if (component instanceof UserInterface) {
                remove(component);
            }
        }
    }

    public boolean checkSignOn() {
        if (null == getEmployee()) {
            SignOnPanel ui = null;
            try {
                ui = getSignOnUserInterface();
                ui.showDialog(Application.getInstance());
            } catch (Exception e) {
                Application.reportException(e);
                return false;
            }
            if (ui.isCancelled() || null == getEmployee()) {
                return false;
            }
        }
        return true;
    }
}
