/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo.ui;

import ianywhere.demo.proxy.Proxy;

import javax.swing.*;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import java.awt.*;
import java.io.PrintWriter;
import java.io.StringWriter;

/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 29-Mar-2003
 * Time: 10:05:43 AM
 */
public class HelpPanel extends JPanel implements UserInterface {
    private static final String LINK_START_PROXY = "start";
    private JEditorPane _htmlView = new JEditorPane();
    public static final String LINK_START_DEMO = "demo";

    public HelpPanel() {
        super();
        initialize();
    }

    public void showWelcome() {
        _htmlView.setText(writeWelcomeHtml());
    }

    private void initialize() {
        writeWelcomeHtml();
        setLayout(new BorderLayout());
        JScrollPane jsp = new JScrollPane(_htmlView);
        add(jsp, BorderLayout.CENTER);
        _htmlView.setEditable(false);
        _htmlView.setContentType("text/html");
        showWelcome();
        _htmlView.addHyperlinkListener(new HyperlinkListener() {
            public void hyperlinkUpdate(HyperlinkEvent e) {
                if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                    String description = e.getDescription();
                    if (LINK_START_PROXY.equalsIgnoreCase(description)) {
                        startProxy();
                    } else if (LINK_START_DEMO.equalsIgnoreCase(description)) {
                        Application app = Application.getInstance();
                        if (app.checkSignOn()) {
                            app.newOrder();
                        }
                    }
                }
            }
        });
    }

    private void startProxy() {
        Proxy proxy = Proxy.getInstance();
        proxy.showConsole();
        try {
            proxy.start();
        } catch (Exception e) {
            Application.reportException(e);
        }
    }

    private String writeWelcomeHtml() {
        StringWriter sw = new StringWriter();
        PrintWriter p = new PrintWriter(sw);
        p.println("<html><body>");
        p.println("<p>");
        p.println("This is a Swing Java application to show you " +
                "how to use ASA 9.0 SOAP feature. In this demo, " +
                "we use APACHE SOAP Java client to communicate with " +
                "ASA database. The demo is a very simple sales order system. " +
                "You can: ");
        p.println("<ul>");
        p.println("<li>Login as a sales employee");
        p.println("<li>Add an order");
        p.println("<li>Update an order");
        p.println("<li>View order list");
        p.println("<li>Add a customer information");
        p.println("<li>Update a customer information");
        p.println("<li>View customer list");
        p.println("</ul>");
        p.println("<p>");
        p.println("It also shows you:");
        p.println("<ul>");
        p.println("<li>How to keep the data integrity over " +
                "SOAP protocol");
        p.println("<li>How to pass complex Java datatypes");
        p.println("</UL>");
        p.println("The <b>Order</b>, <b>XMLSerializer</b> Java classes and " +
                "stored procedure <b>SP_SOAP_UPDATE_ORDER_FROM_XML</b>" +
                " show a solution to " +
                "these two points.");
        p.println("<p>");
        p.println("To monitor the SOAP message, you can " +
                "<a href='" + LINK_START_PROXY + "'>start the SOAP proxy</a> from here." +
                "The proxy seats between this client and SOAP server and captures " +
                "the SOAP messages.");
        p.println("<p>");
        p.println("<b>Notice:</b>If you want to use the proxy, you need to " +
                "start your ASA HTTP listening port at 8080 and make sure " +
                "the port 80 is free for the proxy application.");
        p.println("</p>");
        p.println("<p>");
        p.println("This Java application uses APACHE SOAP (2.3.1) " +
                "library to access SOAP service. The following jar files, " +
                "which are included in the demo files, are needed to run the demo. ");
        p.println("<UL>");
        p.println("<Li>soap.jar -- Apache SOAP jar");
        p.println("<Li>mailapi.jar -- Java Mail API jar");
        p.println("<Li>activation.jar -- JavaBeans Activation Framework jar");
        p.println("</UL>");
        p.println("<p>");
        p.println("<b>Notice:</b>You also need JDK 1.4 or higher to run the demo.</p>");
        p.println("<p>The <b>login password</b> is same as the employee's first name and " +
                "case sensitive.");
        p.println("<p>");
        p.println("<center><b><a href='demo'>Let's start the demo</a></b></center>");
        p.println("</body></html>");
        return sw.toString();
    }
}
