/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo.ui;

import javax.swing.table.AbstractTableModel;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 1-Apr-2003
 * Time: 4:39:09 PM
 */
public class ObjectTableModel extends AbstractTableModel {
    protected List _data = new ArrayList();
    private String[] _columnNames;
    private Class[] _columnTypes;
    private Class _objectClass;
    private StringBuffer _buf = new StringBuffer();

    public ObjectTableModel(Class objectClass) {
        _objectClass = objectClass;
    }

    public ObjectTableModel(Class objectClass, String[] columnNames) {
        _objectClass = objectClass;
        _columnNames = columnNames;
    }

    public Class[] getColumnTypes() {
        return _columnTypes;
    }

    public void setColumnTypes(Class[] columnTypes) {
        _columnTypes = columnTypes;
    }

    public String[] getColumnNames() {
        return _columnNames;
    }

    public void setColumnNames(String[] columnNames) {
        _columnNames = columnNames;
    }

    public Object[] getObjects() {
        Object[] items = new Object[_data.size()];
        _data.toArray(items);
        return items;
    }

    public String getColumnName(int column) {
        return _columnNames[column];
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return getColumnTypes() != null;
    }

    public void addItem(Object item) {
        _data.add(item);
        int rowIndex = _data.size() - 1;
        fireTableRowsInserted(rowIndex, rowIndex);
    }

    public void deleteItem(int index) {
        _data.remove(index);
        fireTableRowsDeleted(index, index);
    }

    public Object getItem(int index) {
        return _data.get(index);
    }

    public int getRowCount() {
        return _data.size();
    }

    public int getColumnCount() {
        return _columnNames.length;
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        if (rowIndex >= 0 && rowIndex < _data.size()) {
            Object item = _data.get(rowIndex);
            String methodName = findMethodName(columnIndex, "set");
            try {
                Class[] parameterTypes = new Class[] {_columnTypes[columnIndex]};
                Method method = _objectClass.getMethod(methodName, parameterTypes);
                method.invoke(item, new Object[] {aValue});
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        if (rowIndex >= 0 && rowIndex < _data.size()) {
            Object item = _data.get(rowIndex);
            try {
                String methodName = findMethodName(columnIndex, "get");
                Method method = _objectClass.getMethod(methodName, null);
                Object o = method.invoke(item, null);
                return null == o ? o : o.toString();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    private String findMethodName(int columnIndex, String getOrSet) {
        String propertyName = _columnNames[columnIndex];
        _buf.setLength(0);
        _buf.append(getOrSet);
        _buf.append(Character.toUpperCase(propertyName.charAt(0)));
        _buf.append(propertyName.substring(1));
        String methodName = _buf.toString();
//        System.out.println("methodName = " + methodName);
        return methodName;
    }
}
