/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo.ui;

import ianywhere.demo.types.Customer;
import ianywhere.demo.types.Order;
import ianywhere.demo.types.OrderItem;
import ianywhere.demo.types.Product;

import javax.swing.*;
import javax.swing.table.TableColumnModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 27-Mar-2003
 * Time: 11:23:11 PM
 */
public class OrderEntry extends JPanel implements UserInterface {
    private OrderItemModel _model = new OrderItemModel();
    private JComboBox _jcbCustomer;
    private JComboBox _jcbRegion;
    private DateField _jtDate;
    private JTextField _jtId;
    private JCheckBox _jcViaXML = new JCheckBox("Send Order via XML stream, this option uses new XML feature in 9.0!");
    private JTable _jtItems;
    private JTextField _jtSales;
    private static String[] _columnNames = new String[]{"Product", "Quantity", "Ship_date"};

    public OrderEntry() throws Exception {
        initialize();
    }

    public OrderItem[] getOrderItems() {
        return _model.getItems();
    }

    public void newOrder() {
        _jtId.setText("");
        _jtSales.setText(Application.getInstance().getEmployee().toString());
        int rowCount = _model.getRowCount();
        for (int i = 0; i < rowCount; i++) {
            _model.deleteItem(0);
        }
    }

    public Order getOrder() {
        Order o = new Order();
        Customer customer = (Customer) _jcbCustomer.getSelectedItem();
        o.setCust_id(customer.getId());
        o.setRegion((String) _jcbRegion.getSelectedItem());
        o.setSales_rep(Application.getInstance().getEmployee().emp_id);
        o.setOrder_date(_jtDate.getText());
        o.setOrderItems(getOrderItems());
        o.setId(_jtId.getText());
        return o;
    }

    public void setOrderId(String id) {
        _jtId.setText(id);
    }

    private void initialize() throws Exception {
        System.out.println("OrderEntry.initialize");
        setLayout(new BorderLayout());
        JLabel jlId = new JLabel("ID:");
        JLabel jlSales = new JLabel("Sales:");
        JLabel jlDate = new JLabel("Date:");
        JLabel jlCustomer = new JLabel("Customer:");
        JLabel jlRegion = new JLabel("Region:");

        _jtId = new JTextField(10);
        _jtId.setEditable(false);
        JTextField jtSales = new JTextField(10);
        _jtDate = new DateField();
        JButton jbSave = new JButton("Save Order");
        jbSave.setMnemonic('S');
        _jcbRegion = new JComboBox(new String[]{"Eastern", "Central", "Western", "South", "Canada"});
        final WholeNumberField jtNumber = new WholeNumberField(1, 10);

        _jtItems = new JTable(_model);
        _jtItems.setRowHeight(30);
        Product[] products = Product.getProducts();
        JComboBox combProduts = new JComboBox(products);
        Customer[] customers = Customer.getCustomers();
        _jcbCustomer = new JComboBox(customers);
        TableColumnModel columnModel = _jtItems.getColumnModel();
        columnModel.getColumn(0).setCellEditor(new OrderItemEditor(combProduts));
        columnModel.getColumn(1).setCellEditor(new ItemQuantityEditor(jtNumber));
        columnModel.getColumn(2).setCellEditor(new DateEditor(new DateField()));
        JButton jbAdd = new JButton("Add Item");
        JButton jbDelete = new JButton("Delete Item");
        _jtSales = jtSales;
        _jtSales.setEditable(false);
        jtSales.setText(Application.getInstance().getEmployee().toString());

        jbAdd.setMnemonic('A');
        jbDelete.setMnemonic('D');
        JPanel jpOrder = new JPanel(new GridBagLayout());
        JPanel jpActions = new JPanel(new GridBagLayout());

        jpOrder.setBorder(BorderFactory.createTitledBorder("Order Information"));
        // id
        GridBagConstraints c = new GridBagConstraints(1, 1, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpOrder.add(jlId, c);
        c = new GridBagConstraints(2, 1, 1, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpOrder.add(_jtId, c);

        // sales
        c = new GridBagConstraints(3, 1, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpOrder.add(jlSales, c);
        c = new GridBagConstraints(4, 1, 1, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpOrder.add(jtSales, c);

        // date
        c = new GridBagConstraints(5, 1, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpOrder.add(jlDate, c);
        c = new GridBagConstraints(6, 1, 1, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpOrder.add(_jtDate, c);

        // region
        c = new GridBagConstraints(1, 2, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpOrder.add(jlRegion, c);
        c = new GridBagConstraints(2, 2, 1, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpOrder.add(_jcbRegion, c);

        // save button
        c = new GridBagConstraints(6, 2, 1, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpOrder.add(jbSave, c);
        jbSave.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                saveOrder();
            }
        });

        // checkbox viaXML
        c = new GridBagConstraints(1, 3, 6, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpOrder.add(_jcViaXML, c);
        _jcViaXML.setMnemonic('X');

        // customer
        c = new GridBagConstraints(3, 2, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpOrder.add(jlCustomer, c);
        c = new GridBagConstraints(4, 2, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpOrder.add(_jcbCustomer, c);

        // add
        c = new GridBagConstraints(1, 1, 1, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpActions.add(jbAdd, c);
        jbAdd.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                _model.addItem();
            }
        });

        // delete
        c = new GridBagConstraints(2, 1, 1, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        jpActions.add(jbDelete, c);
        jbDelete.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int selectedRow = _jtItems.getSelectedRow();
                if (selectedRow >= 0) {
                    _model.deleteItem(selectedRow);
                }
            }
        });
        // big panels
        c = new GridBagConstraints(1, 2, 1, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                new Insets(2, 2, 2, 2), 2, 2);
        add(jpOrder, BorderLayout.NORTH);

        JScrollPane jsp = new JScrollPane(_jtItems);
        add(jsp, BorderLayout.CENTER);
        add(jpActions, BorderLayout.SOUTH);
    }

    public void setOrder(Order order) throws Exception {
        newOrder();
        _jtId.setText(order.id);
        _jcbRegion.setSelectedItem(order.region);
        _jtDate.setText(order.order_date);
        Customer customer = Customer.getCustomer(order.cust_id);
        _jcbCustomer.setSelectedItem(customer);
        try {
            OrderItem[] orderItems = order.getOrderItems();
            for (int i = 0; i < orderItems.length; i++) {
                OrderItem orderItem = orderItems[i];
                Product product = Product.getProduct(orderItem.getProd_id());
                orderItem.setProduct(product);
                _model.addItem(orderItem);
            }
        } catch (Exception ignore) {
            ignore.printStackTrace();
            // ASA SOAP BUG! Empty resultset causes invalid
            // SOAP response
        }
    }

    private void saveOrder() {
        Order order = getOrder();
        if (order.hasItems()) {
            System.out.println("order = " + order);
            try {
                if (!_jcViaXML.isSelected()) {
                    Order.saveOrder(order);
                    setOrderId(order.getId());
                } else {
                    int orderId = Order.saveViaXml(order);
                    setOrderId(String.valueOf(orderId));
                }
            } catch (Exception e) {
                Application.reportException(e);
            }
        }
    }

    private class OrderItemModel extends ObjectTableModel {
        private static final int COLUMN_NUMBER_SHIPDATE = 2;

        public OrderItemModel() {
            super(OrderItem.class, _columnNames);
            setColumnTypes(new Class[]{Product.class, String.class, String.class});
        }

        public OrderItem[] getItems() {
            OrderItem[] items = new OrderItem[_data.size()];
            _data.toArray(items);
            return items;
        }

        public void addItem() {
            OrderItem orderItem = new OrderItem();
            super.addItem(orderItem);
        }

        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            if (columnIndex == COLUMN_NUMBER_SHIPDATE) {
                String ship_date = (String) aValue;
                if (DateField.check(ship_date)) {
                    super.setValueAt(ship_date, rowIndex, columnIndex);
                } else {
                    Toolkit.getDefaultToolkit().beep();
                }
            } else {
                super.setValueAt(aValue, rowIndex, columnIndex);
            }
        }
    }

    private class OrderItemEditor extends DefaultCellEditor {
        public OrderItemEditor(JComboBox comboBox) {
            super(comboBox);
        }
    }

    private class DateEditor extends DefaultCellEditor {
        public DateEditor(final JTextField textField) {
            super(textField);
            setClickCountToStart(1);
        }
    }

    private class ItemQuantityEditor extends DefaultCellEditor {
        public ItemQuantityEditor(final JTextField textField) {
            super(textField);
        }
    }

}
