/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo.ui;

import ianywhere.demo.types.Customer;
import ianywhere.demo.types.Employee;
import ianywhere.demo.types.Order;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 29-Mar-2003
 * Time: 9:23:56 PM
 */
public class OrderView extends JPanel implements UserInterface {
    private OrderModel _model;
    private JTable _view = new JTable(_model);
    private String[] _columnNames = new String[]{
        "id", "cust_id", "region", "order_date"
    };

    public OrderView() {
        super();
        initialize();
    }

    public void retrieve(Employee employee) throws Exception {
        System.out.println("OrderView.retrieve");
        System.out.println("employee = " + employee);
        String emp_id = employee.getEmp_id();
        Order[] orders = Order.getOrdersBySales(emp_id);
        _model = new OrderModel();
        for (int i = 0; i < orders.length; i++) {
            Order order = orders[i];
            _model.addItem(order);
        }
        _view.setModel(_model);
    }

    private void initialize() {
        setLayout(new BorderLayout());
        setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        _model = new OrderModel();
        _view.setRowHeight(25);
        JScrollPane jsp = new JScrollPane(_view);
        add(jsp, BorderLayout.CENTER);
        _view.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        _view.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    int selectedRow = _view.getSelectedRow();
                    if (selectedRow >= 0) {
                        Order order = (Order) _model.getItem(selectedRow);
                        Application instance = Application.getInstance();
                        instance.editOrder(order);
                    }
                }
            }
        });
        setBorder(BorderFactory.createTitledBorder("Order List"));
    }

    private class OrderModel extends ObjectTableModel {

        public OrderModel() {
            super(Order.class, _columnNames);
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            if (columnIndex == 1) {
                String cust_id = (String) super.getValueAt(rowIndex, columnIndex);
                try {
                    return Customer.getCustomer(cust_id);
                } catch (Exception e) {
                    return cust_id;
                }
            } else {
                return super.getValueAt(rowIndex, columnIndex);
            }
        }
    }
}
