/*##COPYRIGHT
 ***********************************************************************
 *       Copyright (C) 2003, by iAnywhere Solutions, Inc.              *
 *       All rights reserved. iAnywhere Solutions is a trademark       *
 *       of Sybase, Inc.  No part of this software may be              *
 *       reproduced in any form or by any means - graphic,             *
 *       electronic or mechanical, including photocopying,             *
 *       recording, taping or information storage and retrieval        *
 *       systems - except with the written permission of               *
 *       iAnywhere Solutions, Inc.                                     *
 *                                                                     *
 ***********************************************************************
#*/

package ianywhere.demo.ui;

import ianywhere.demo.ISoapServices;
import ianywhere.demo.types.Employee;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

/**
 * Created by iAnywhere Solutions
 * User: huojun
 * Date: 28-Mar-2003
 * Time: 12:01:38 AM
 */
public class SignOnPanel extends JPanel {
    private String _user;
    private String _password;
    private Employee _employee;
    private boolean _cancelled = false;
    private JDialog _dialog = null;
    private JPasswordField _jpfPass = new JPasswordField(15);
    private JComboBox _jcbUser;
    private static final String KEY_ESC = "esc";
    private static final String KEY_ENTER = "enter";

    public static void main(String[] args) throws Exception {
        SignOnPanel sp = new SignOnPanel();
        sp.showDialog(Application.getInstance());
    }

    public SignOnPanel() throws Exception {
        initialize();
    }

    public boolean isCancelled() {
        return _cancelled;
    }

    public String getUser() {
        return _user;
    }

    public Employee getEmployee() {
        return _employee;
    }

    public String getPassword() {
        return _password;
    }

    public void showDialog(Frame owner) {
        if( null == _dialog ) {
            createDialog();
        }
        _dialog.setVisible(true);
    }

    private void setEmployee(Employee employee) {
        _employee = employee;
    }

    private void createDialog() {
        _dialog = new JDialog();
        _dialog.setModal(true);
        _dialog.setTitle("Employee Sign On");
        _dialog.getContentPane().add(this);
        _dialog.setResizable(false);
        _dialog.pack();
        _dialog.setLocationRelativeTo(null);
        _dialog.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
    }

    private void setUser(String user) {
        _user = user;
    }

    private void setPassword(String password) {
        _password = password;
    }

    private void setCancelled(boolean cancelled) {
        _cancelled = cancelled;
    }

    private void initialize() throws Exception {
        JLabel jlUser = new JLabel("User:");
        JLabel jlPassword = new JLabel("Password:");
        JButton jbOK = new JButton("OK");
        JButton jbCancel = new JButton("Cancel");

        Employee[] sales = Application.getInstance().getSoapClient().getSales();
        _jcbUser = new JComboBox(sales);
        InputMap defaultMap = _jpfPass.getInputMap(JComponent.WHEN_FOCUSED);
        defaultMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), KEY_ESC);
        _jpfPass.getActionMap().put(KEY_ESC, new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                doCancel();
            }
        });

        defaultMap = _jcbUser.getInputMap(JComponent.WHEN_FOCUSED);
        defaultMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), KEY_ESC);
        _jcbUser.getActionMap().put(KEY_ESC, new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                doCancel();
            }
        });

        defaultMap = _jpfPass.getInputMap(JComponent.WHEN_FOCUSED);
        defaultMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), KEY_ENTER);
        _jpfPass.getActionMap().put(KEY_ENTER, new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                doSignOn();
            }
        });

        defaultMap = _jcbUser.getInputMap(JComponent.WHEN_FOCUSED);
        defaultMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), KEY_ENTER);
        _jcbUser.getActionMap().put(KEY_ENTER, new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                doSignOn();
            }
        });
        jbCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                doCancel();
            }
        });
        jbOK.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                doSignOn();
            }
        });
        jbCancel.setMnemonic('C');
        jbOK.setMnemonic('O');
        setBorder(BorderFactory.createEmptyBorder(10, 20, 10, 20));
        setLayout(new GridBagLayout());
        // user
        GridBagConstraints c = new GridBagConstraints(1, 1, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        add(jlUser, c);
        // user field
        c = new GridBagConstraints(2, 1, 2, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        add(_jcbUser, c);
        // password
        c = new GridBagConstraints(1, 2, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        add(jlPassword, c);
        // password field
        c = new GridBagConstraints(2, 2, 2, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        add(_jpfPass, c);
        // OK
        c = new GridBagConstraints(2, 3, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        add(jbOK, c);
        // Cancel
        c = new GridBagConstraints(3, 3, 1, 1, 0, 0, GridBagConstraints.EAST, GridBagConstraints.NONE,
                new Insets(2, 2, 2, 2), 2, 2);
        add(jbCancel, c);
    }

    private void doCancel() {
        _dialog.setVisible(false);
        setCancelled(true);
    }

    private void doSignOn() {
        System.out.println("SignOnPanel.doSignOn");
        Employee sales = (Employee) _jcbUser.getSelectedItem();
        setCancelled(false);
        setUser(sales.emp_fname);
        setPassword(new String(_jpfPass.getPassword()));
        if (verify(getUser(), getPassword())) {
            _dialog.setVisible(false);
            Application.getInstance().setEmployee(getEmployee());
        } else {
            showError();
        }
    }

    private boolean verify(String user, String password) {
        System.out.println("SignOnPanel.verify");
        try {
            ISoapServices sc = Application.getInstance().getSoapClient();
            Employee employee = sc.signOn(user, password);
            if (employee.getEmp_id() == null) {
                setEmployee(null);
                return false;
            }
            System.out.println("employee = " + employee);
            setEmployee(employee);
            return true;
        } catch (Exception e) {
            Application.reportException(e);
        }
        return false;
    }

    private void showError() {
        JOptionPane.showMessageDialog(this, "Invalid user or password!");
    }
}
